#include <windows.h>
//#include <math.h>
//#include <float.h>
#include "../MachineInterface.h"
#include "basic.h"
#include "parameter.h"
#include "inertia.h"

#define MAX_TRACKS		0
#define MIN_TRACKS		0
#define MACHINE_NAME	"Amp"
#define SHORT_NAME		"A"
#define VERSION			"0.2"
#define ABOUTTXT MACHINE_NAME" v"VERSION"\n\nbuild: "__DATE__"\n\nMarcin Dabrowski\nhttp://bigyo.untergrund.net\n"

CMachineAttribute const *pAttributes[] = { NULL };
#define numAttributes	0
CMachineParameter const paraVolume = { pt_byte, "Volume", "Volume", 0, 0x80, 0xFF, MPF_STATE, 0x60 };
CMachineParameter const para0dBposition = { pt_byte, "0dBposition", "0dBposition", 1, 0x7F, 0xFF, MPF_STATE, 0x60 };
CMachineParameter const paraPower = { pt_byte, "Shape", "Shape", 0, 0x80, 0xFF, MPF_STATE, 0x30 };
CMachineParameter const *pParameters[] = {	&paraVolume, &para0dBposition, &paraPower };
#define numGlobalParameters		3
#define numTrackParameters		0
////////////////////////////////////////////////////////////////////////////////////
CMachineInfo const MacInfo = 
{
	MT_EFFECT,				// Machine type // MT_GENERATOR or MT_EFFECT
	MI_VERSION,				// Machine interface version
	MIF_STEREO_EFFECT,		// Machine flags
	MIN_TRACKS,				// min tracks
	MAX_TRACKS,				// max tracks
	numGlobalParameters,	// numGlobalParameters
	numTrackParameters,		// numTrackParameters
	pParameters,			// pointer to parameter stuff
	numAttributes,			// numAttributes
	pAttributes,			// pointer to attribute stuff
	MACHINE_NAME,			// Full Machine Name
	SHORT_NAME,				// Short name
	"Marcin Dabrowski",		// Author name
	"&About..."				// Right click menu commands
};
/////////////////////////////////////////////////////////////////////////////////////
#pragma pack(1)                        
class CGval
{
	public:
		byte volume;
		byte _0dbposition;
		byte power;
};
#pragma pack()

class mi;
class miex : public CMachineInterfaceEx { public: mi *pmi; };
class mi : public CMachineInterface
{
	public:
            mi();
			virtual ~mi() {}
			virtual void Tick();
            virtual void Init(CMachineDataInput * const pi);
            virtual bool Work(float *psamples, int numsamples, int const mode);
            virtual char const *DescribeValue(int const param, int const value);
			virtual void Command(int const i);
			inline double seconds2samples(double sec) {return  (((double) pMasterInfo->SamplesPerSec) * sec  ) ;}
			miex ex;
			CMachine * pmac;
	private:
			CGval Gval;
	public:
			parameterPowLog	Volume;
			parameterLin	Power;
			
			OnePoleInertia<double> gain;
			int _0db;
};
//////////////////////
mi::mi() 
{
		ex.pmi = this;
		GlobalVals = &Gval;
		TrackVals = NULL;
		AttrVals = NULL;
}
void mi::Init(CMachineDataInput * const pi)
{
		pCB->SetMachineInterfaceEx(&ex);
		pmac =  pCB->GetThisMachine();
		Power.init(&paraPower, 0.0, 5.0);
		Volume.init(&paraVolume, 0.0, DBToAmplitude(10.0), 1.0, paraVolume.DefValue, Power(paraPower.DefValue));  
		gain.init(Volume(paraVolume.DefValue));
		gain.setTime( seconds2samples(0.01) );
		
}
void mi::Tick()
{

	bool updateslider=false;
	if  (Volume.update(Gval.volume))
	{
		gain = Volume();
	}
	if (Gval._0dbposition != para0dBposition.NoValue)
	{
		updateslider=true;
		_0db = Gval._0dbposition;
	}
	if (Power.change(Gval.power))
	{
		updateslider=true;
	}
	if (updateslider)
	{
		Volume.slope_set(1.0, _0db, Power());
		gain = Volume();
		pCB->InvalidateParameterValueDescription(pmac,0);
	}
}

bool mi::Work(float *psamples, int numsamples, int const mode)
{
		if (mode==WM_NOIO)
			return false;
		if (mode==WM_READ)		// <thru>
			return true; 
		if (mode==WM_WRITE) 	// 
			return false;

		//mode==WM_READWRITE
		for (int i=0; i<numsamples; i++)
		{
			*psamples++ *= gain;
			*psamples++ *= gain;
			gain.tick();
		}
		return true;
}
char const *mi::DescribeValue(int const param, int const value)
{
		static char txt[16];
		switch(param)
		{
				case 0:		Volume.describe_db(txt,value); break;
				case 1:		sprintf_s(txt, "%d", value); 	break;
				case 2:		Power.describe(txt,value,""); break;
		}
		return txt;
}
void mi::Command(int const i)
{
		switch (i)
		{
				case 0:
						MessageBox(NULL,ABOUTTXT,MACHINE_NAME,MB_OK|MB_ICONINFORMATION);
						break;
				default:
						break;
		}
}
DLL_EXPORTS