// http://bigyo.untergrund.net

#pragma once

#include <math.h>

double const oolog2 = 1.0 / log(2.0);
double const logof2 = log(2.0);

template <typename type> struct XY { type X,Y; };
template <typename type> struct xy { type x,y; };

template <typename type> type clamp(type value, type min, type max) { return (value < min) ? min : (value > max) ? max : value; };

template <typename type> inline const type& _max(const type& num1, const type& num2) { return  num1 < num2 ? num2 : num1; }; 
template <typename type> inline const type& _min(const type& num1, const type& num2) { return  num1 > num2 ? num2 : num1; }; 

inline double	DBToAmplitude(double const dB) { return pow(10.0 , dB * 0.05 ); }
inline float	DBToAmplitude(float const dB) { return powf(10.0f , dB * 0.05f ); }

inline double	DBToPower(double const dB) { return pow( 10.0 , dB * 0.1 ); }
inline float	DBToPower(float const dB) { return powf( 10.0f , dB * 0.1f ); }

inline double	AmplitudeToDB(double const amplitude) { return 20.0*log10(amplitude); }
inline float	AmplitudeToDB(float const amplitude) { return 20.0f*log10f(amplitude); }

inline double	PowerToDB(double const x2) { return 10.0*log10(x2); }
inline float	PowerToDB(float const x2) { return 10.0f*log10f(x2); }

inline double	ScaleLin(double x, double y_min, double y_max) { return x * (y_max - y_min) + y_min ; }
inline float	ScaleLin(float x, float y_min, float y_max) { return x * (y_max - y_min) + y_min ; }

inline double	ScaleLin(double x, double x_min, double x_max, double y_min, double y_max) { return  ScaleLin( (x - x_min) / (x_max - x_min), y_min, y_max); }
inline float	ScaleLin(float x, float x_min, float x_max, float y_min, float y_max) { return  ScaleLin( (x - x_min) / (x_max - x_min), y_min, y_max); }

inline double	ScaleLog(double x, double y_min, double y_max) { return y_min * pow (y_max/y_min, x); }
inline float	ScaleLog(float x, float y_min, float y_max) { return y_min * powf (y_max/y_min, x); }

inline double	ScaleLog(double x, double x_min, double x_max, double y_min, double y_max) { return ScaleLog( (x-x_min)/(x_max-x_min) , y_min , y_max ); }
inline float	ScaleLog(float x, float x_min, float x_max, float y_min, float y_max) { return ScaleLog( (x-x_min)/(x_max-x_min) , y_min , y_max ); }

inline double	ScalePow(double x, double y_min, double y_max, double power) { return ScaleLin(pow(x,power),y_min,y_max); }
inline float	ScalePow(float x, float y_min, float y_max, float power) { return ScaleLin(pow(x,power),y_min,y_max); }

inline double	ScalePow(double x, double x_min, double x_max, double y_min, double y_max, double power) { return ScalePow( (x-x_min)/(x_max-x_min) , y_min , y_max, power ); }
inline float	ScalePow(float x, float x_min, float x_max, float y_min, float y_max, float power) { return ScalePow( (x-x_min)/(x_max-x_min) , y_min , y_max, power ); }

double ScaleLinLog(double x, double x_min, double x_max, double y_min, double y_max, double slope)
{
	double y;
	double x01 = (x-x_min) / (x_max - x_min);
	if (slope <= 0.5) y =  x01  * pow( 2.0 * slope, 2.0 - 2.0*x01 ); 		
	else y = 1.0 + (x01-1.0) * pow(2.0 - 2.0 * slope, 2.0 * x01 );
	return y * fabs(y_max - y_min) + y_min;
}

double ScaleSqrLog(double x, double x_min, double x_max, double y_min, double y_max, double base)
{
	double y;
	double x01 = (x-x_min) / (x_max - x_min);
	y =  x01*x01  * pow( 2.0 * base, 2.0 - 2.0*x01 ); 		
	return y * (y_max - y_min) + y_min;
}
double ScalePowLog(double x, double x_min, double x_max, double y_min, double y_max, double base,double power)
{
	double y;
	double x01 = (x-x_min) / (x_max - x_min);
	y =  pow(x01,power)  * pow( 2.0 * base, 2.0 - 2.0*x01 ); 		
	return y * (y_max - y_min) + y_min;
}


inline float datan(float x) { return 1.0f/(x*x+1.0f); }
inline float fast_tanh(float x) { if( x < -3.0f ) return -1.0f; else if( x > 3.0f ) return 1.0f; else return x * ( 27.0f +  x * x ) / ( 27.0f + 9.0f * x * x ); }
inline float fast_log2 (float val) {  int * const  exp_ptr = reinterpret_cast <int *> (&val); int x = *exp_ptr; const int log_2 = ((x >> 23) & 255) - 128; x &= ~(255 << 23); x += 127 << 23; *exp_ptr = x; val = ((-1.0f/3) * val + 2) * val - 2.0f/3; return (val + log_2); }
inline float fast_log (const float &val) { return (fast_log2 (val) * 0.69314718f); } 
inline float fast_log10 (const float &val){ return (fast_log2 (val) * 0.3010299957f); } 
inline float fast_acos (float x) { float xx = x*x; float y = 1.570796328f - x*( 1.0f + xx*( 0.092f + xx*( 0.662f + xx*( -1.362f + xx*( 1.062f))))); if(y<0.0f) return 0.0f; if(y>3.14159265f) return 3.14159265f; return y;}
inline float sinc(float x) { if  ( fabs(x) < 0.018581361 ) { float x2 = x*x; return 1.0 - x2*(1.0/6.0)+x2*x2*(1.0/120.0); } else { return sin(x)/x ; } }
inline int nextpow2(int i) { int p = 1; while(p < i) p <<= 1; return p; }

void Float32toInt32(float* src, void* dest, long numsamples) { int* destination = (int*)dest;   while (numsamples--)	   *destination++ = (int)(*src++ * 2147483647.0f) ;}
void Int32toFloat32(void* src, float* dest, long numsamples){ int* source = (int*) src;	while (numsamples--)		*dest++ = (float) (*source++ * (1.0f / 2147483647.0f));}
