#ifndef __INERTIA_H
#define __INERTIA_H

#include <math.h>

template <class datatype>
class LinInertia
{
public:
	LinInertia() { out = in = 0; }
	virtual ~LinInertia() {}
	void init(datatype val) {	out = in = val ;}
	void reset() { out=in; }
	void setTime(datatype time)  { this->time = time; delta = (in-out) / time; }	// w ilu krokach ma sie zmienic z out do in
	void change(datatype in) { this->in = in; delta = (in-out) / time; }
	void operator= (datatype in) { change(in);}
	inline bool tick();
	datatype out;
	datatype in;			// out=actual, in=target
	operator datatype() { return out;}
private:
	datatype delta;
	datatype time;
};

template <class datatype>
class LogInertia	
{
public:
	LogInertia() 	{ k = time = in = out = 1.0 ; }
	virtual ~LogInertia() {}
	void init(datatype val) { out = in = val; }
	void reset() { out=in; }
	void setTime(datatype time) { this->time = time; k = pow((double)in/(double)out, 1.0/(double)time); } // w ilu krokach ma sie zmienic z out do in
	//void change(datatype in) 	{if (in) { this->in = in; k = pow((double)in/(double)out, 1.0/(double)time); }}
	void operator= (const datatype& val) { if (val) { in = val; k = pow((double)in/(double)out, 1.0/(double)time); } }
	inline bool tick();
	datatype out;					// out=actual, in=target
	datatype in;
	operator datatype() { return out;}
private:
	datatype k;
	datatype time;
};

template <class type>
class OnePoleInertia 	// out[n] = b * in[n] - (b-1) * out[n-1]
{
public:
	OnePoleInertia() {	b0 = 1.0 ; a1 = 0.0; out = 0.0 ;}
	~OnePoleInertia() {}
	void init(type val) { out = in = val; }
	void reset() { out=in; }
	void setTime(type time) { b0 = 1.0 / (time*oolog2 + (1.0-1.0/PI)); a1 = b0 - 1.0; }
	//void setPole(type pole) { b = 1.0 - pole ; a = pole ;}
	//void change(type in) {this->in = in;}
	void operator= (const type& in) { this->in = in;}
	inline void tick() {	out = b0 * in - a1 * out ;} //v = b0 * in - a1 * v ; 	out = b0 * v - a1 * out ;}
	type out;					// out=actual, in=target
	type in;
	operator type() { return out;}
private:	
	//type v;
	type b0, a1;
};

/////////////////////////////////////////////////////////////////////////////////////
template <class datatype>
bool LinInertia<datatype> :: tick()
{
	if (out == in)
		return false;
	else 
	{
		if (out > in) { out += delta; if (out < in) out = in; }
		else          { out += delta; if (out > in) out = in; }
		return true;
	}
}

template <class datatype>
bool LogInertia<datatype> :: tick()
{
	if (out == in)
		return false;
	else
	{
		if (out > in) { out *= k; if (out < in) out = in; }
		else          { out *= k; if (out > in) out = in; }
		return true;
	}
}



#endif 
